package de.hftstuttgart.unifiedticketing.systems.gitlab;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import de.hftstuttgart.unifiedticketing.exceptions.AssertionException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;

import java.util.regex.Pattern;

public class GitlabTicketSystemBuilderTest
{
    public GitlabTicketSystemBuilder instance;

    @BeforeEach
    public void initBeforeEach()
    {
        instance = Mockito.spy(new GitlabTicketSystemBuilder());
    }

    @Test
    public void testInit()
    {
        assertAll(
            () -> assertNull(instance.apiKey),
            () -> assertEquals("v4", instance.apiVersion),
            () -> assertEquals(-1, instance.projectId),
            () -> assertTrue(instance.https)
        );
    }

    @Test
    public void testWithApiKey()
    {
        String expected = "asoashdboahasjhdfbeuazhvfef4q34v3h4v435v2";
        instance.withApiKey(expected);
        assertEquals(expected, instance.apiKey);
    }

    @Test
    public void testWithHttp()
    {
        instance.https = true;
        instance.withHttp();

        assertFalse(instance.https);
    }

    @Test
    public void testWithHttps()
    {
        instance.https = false;
        instance.withHttps();
        assertTrue(instance.https);
    }

    @Test
    public void testWithProjectIdByInt()
    {
        int expected = 255254123;
        instance.withProjectId(expected);

        assertEquals(expected, instance.projectId);
    }

    @Test
    public void testWithProjectIdByString()
    {
        int expected = 23523423;
        instance.withProjectId(String.valueOf(expected));
        verify(instance, times(1)).withProjectId(expected);

        assertThrows(AssertionException.class, () -> instance.withProjectId("asdf3asd"));
    }

    @Test
    public void testBuild()
    {
        instance.withBaseUrl("blabla.tld");
        assertThrows(AssertionException.class, () -> instance.build());

        instance = new GitlabTicketSystemBuilder();
        instance.withProjectId(42);
        assertThrows(AssertionException.class, () -> instance.build());

        instance = new GitlabTicketSystemBuilder();
        String baseUrl = "gitlab.some.tld";
        int projectId = 42;
        String apiKey = "aaesfsef32qqfq3f";
        String regex = "^%s://%s/api/v4/projects/%d$";
        Pattern pattern = Pattern.compile(
            String.format(regex, "https", baseUrl, projectId));

        instance.withBaseUrl(baseUrl).withProjectId(projectId);
        GitlabTicketSystem system = instance.build();
        assertTrue(pattern.matcher(system.baseUrl).matches());

        pattern = Pattern.compile(String.format(regex, "http", baseUrl, projectId));
        instance.withHttp().withApiKey(apiKey);
        system = instance.build();
        assertTrue(pattern.matcher(system.baseUrl).matches());
        Assertions.assertEquals(apiKey, system.apiKey);
    }
}
